#  Copyright 2014 Klaudiusz Staniek
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.

"""
 fiblary.rooms
 ~~~~~~~~~~~~~~

 Home Center Room Manager Implementation
"""

from fiblary3.client.v4 import base


class Controller(base.CommonController):
    RESOURCE = 'rooms'
    API_PARAMS = ('id', 'sectionID')
